TranslationLanguage=Polski

"Remote to local path mappings.~~Examples:~/share=\\pch\share~/var/downloads/music=Z:\music"="Mapowanie ścieżek zdalnych do lokalnych.~~Przykłady:~/share=\\pch\share~/var/downloads/music=Z:\music"
%d x %s (have %d)=%d x %s (pobrano %d)
%ds=%ds
%s (%d hashfails)=%s (%d błędów bufora)
%s (%s done)=%s (%s pobrane)
'%s' has finished downloading=Zakończono pobieranie '%s'
%s%s%d downloading, %d seeding%s%s, %s=%s%s%d pobieranie, %d seedowanie%s%s, %s
&All=&Wszystkie
&Close=&Zamknij
&Help=&Pomoc
&Ignore=&Ignoruj
&No=&Nie
&OK=&OK
&Open=&Otwórz
&Retry=&Ponów
&Save=&Zapisz
&Unlock=&Odblokuj
&Yes=&Tak
/s=/s
Abort=Przerwij
About=O programie
Active=Aktywne
Add new torrent=Dodaj nowy torrent
&Add torrent=Dodaj torrent
Added on=Dodane o
Are you sure to remove torrent '%s' and all associated DATA?=Czy na pewno chcesz usunąć plik torrent '%s' oraz wszystkie pobrane dane?
Are you sure to remove torrent '%s'?=Czy na pewno chcesz usunąć plik torrent '%s'?
Authentication=Autoryzacja
b=b
Cancel=Anuluj
Client=Klient
Close to tray=Zamknij do tray'a
Comment=Komentarz
Completed on=Zakończono o
Completed=Ukończono
Confirmation=Potwierdzenie
connected=połączono
Connecting to daemon=Łączenie do demona
Connection error occurred=Wystąpił błąd połączenia
Copy=Kopiuj
Country=Kraj
Created on=Utworzono o
D: %s/s=D: %s/s
Destination folder=Folder docelowy
Disconnected=Rozłącz
Donate to support further development=Darowizna na wsparcie dalszego rozwoju
Donate via PayPal,WebMoney,Credit card=Dotuj przez PayPal,WebMoney,Credit card
Done=Postęp
Don't download=Nie pobieraj
Down limit=Limit pobierania
Down speed=Pobieranie
Down=W dół
Download complete=Zakończono pobieranie
Download speed=Prędkość pobierania
Downloaded=Pobrane
Downloading=Pobieranie
Enable DHT=Włącz DHT
Enable Peer Exchange=Włącz Peer Exchange
Enable port forwarding=Włącz przekierowywanie portów
Encryption disabled=Szyfrowanie wyłączone
Encryption enabled=Szyfrowanie włączone
Encryption required=Szyfrowanie wymagane
Encryption=Szyfrowanie
Error=Błąd
ETA=ETA
E&xit=Wyjdź
File name=Nazwa pliku
Files=Pliki
Finished=Ukończono
Flag images archive is needed to display country flags.~Download this archive now?=Obrazki flag są potrzebne, aby wyświetlać je w karcie peerów.~Czy chcesz je teraz pobrać?
Flags=Flagi
GB=GB
General=Ogólne
Geo IP database is needed to resolve country by IP address.~Download this database now?=Baza Geo IP jest potrzebna aby wykryć kraj peer'a.~Czy chcesz ją teraz pobrać?
Global bandwidth settings=Ogólne ustawienia prędkości
Global peer limit=Ogólny limit peerów
Hash=Hash
Have=Pobrał
Hide=Ukryj
High priority=Wysoki priorytet
high=Wysoki
Host=Host
in swarm=w kolejce
Inactive=Nieaktywne
Incoming port is closed. Check your firewall settings=Port połączeń przychodzących jest zablokowany. Sprawdź ustawienia swojego firewalla.
Incoming port tested successfully=Test portu połączeń przychodzących wypadł pomyślnie
Incoming port=Port połączeń przychodzących
Information=Informacja
KB/s=KB/s
KB=KB
Language=Język
Last active=Ostatnio aktywny
License=Licencja
Low priority=Niski priorytet
low=niski
Max peers=Maksymalna ilość peerów
Maximum download speed=Maksymalna prędkość pobierania
Maximum upload speed=Maksymalna prędkość wysyłania
MB=MB
Minimize to tray=Schowaj do tray'a
Name=Nazwa
No host name specified=Nie zdefiniowano hostname
No proxy server specified=Nie zdefiniowano serwera proxy
No to all=Nie na wszystko
Normal priority=Normalny priorytet
normal=normalny
of=z
Open containing folder=Otwórz folder zawierający
Open=Otwórz
Password=Hasło
Paths=Ścieżki
Peer limit=Limit peerów
Peers=Peery
Pieces=Części
Port=Port
Priority=Priorytet
Properties=Właściwości
Proxy password=Hasło proxy
Proxy port=Port proxy
Proxy server=Serwer proxy
Proxy user name=Nazwa użytkownika proxy
Ratio=Ratio
Reconnect in %d seconds=Ponawianie próby za %d sekund
Remaining=Pozostało
Remote host=Zdalny host
Remove torrent and Data=Usuń torrenta razem z danymi
Remove torrent=Usuń torrenta
Remove=Usuń
Resolve country=Sprawdzaj kraj
Resolve host name=Sprawdzaj hostname
seconds=sekund
Seed ratio=Ratio seedowania
Seeding=Seedowanie
Seeds=Seedy
Select a .torrent to open=Wybierz plik .torrent aby otworzyć
Select all=Zaznacz wszystko
Select none=Odznacz wszystko
Setup columns=Ustawienia kolumn
Share ratio=Ratio współdzielenia
Show country flag=Wyświetlaj flagi państw
Show=Pokaż
Size=Rozmiar
skip=pomiń
Start all torrents=Rozpocznij pobieranie wszystkiego
Start torrent=Rozpocznij pobieranie
Start=Start
Status=Stan
Stop all torrents=Zatrzymaj wszystkie torrenty
Stop all=Zatrzymaj wszystko
Stop torrent=Zatrzymaj torrent
Stop=Stop
Stopped=Zatrzymane
TB=TB
Test port=Testuj port
T&ools=Narzędzia
Torrent contents=Treść torrenta
Torrent properties=Właściwości torrenta
Torrent verification may take a long time.~Are you sure to start verification of torrent '%s'?=Weryfikacja torrenta może trwać bardzo długo.~Czy na pewno chcesz zweryfikować torrent '%s'?
&Torrent=Torrent
Torrents (*.torrent)|*.torrent|All files (*.*)|*.*=Torrenty (*.torrent)|*.torrent|Wszystkie pliki (*.*)|*.*
Total size=Łączny rozmiar
Tracker status=Status trackera
Tracker update on=Aktualizacja trackera o
Tracker=Tracker
Trackers=Trackery
Transfer=Transfer
Transmission options=Opcje Transmission
Transmission%s at %s:%s=Transmission%s na %s:%s
Tray icon always visible=Ikonka w trayu zawsze widoczna
Tray icon=Ikonka w trayu
U: %s/s=U: %s/s
Unable to extract flag image=Nie można rozpakować obrazków flag
Unable to get files list=Nie można pobrać listy plików
Unknown=Nieznane
Up limit=Limit wysyłania
Up speed=Wysyłanie
Up=W górę
Update complete=Uaktualnianie zakończone
Update GeoIP database=Uaktualnij bazę GeoIP
Update in=Aktualizacja za
Updating=Aktualizacja
Upload speed=Prędkość wysyłania
Uploaded=Wysłane
User name=Nazwa użytkownika
Verify torrent=Zweryfikuj torrenta
&Verify=Weryfikuj
Verifying=Sprawdzanie
Version %s=Wersja %s
Waiting=Oczekiwanie
Warning=Ostrzeżenie
Wasted=Zmarnowane
Working=Działa
Yes to &All=Tak na &Wszystkie
No tracker=Brak trackera
%s downloaded=%s pobrano
%s of %s downloaded=%s z %s pobrano
%d torrents=%d torrenty
Add .part extension to incomplete files=Dodaj końcówkę .part do plików niedokończonych
Add torrent link=Dodaj link do torrenta
Are you sure to remove %d selected torrents and all their associated DATA?=Czy na pewno chcesz usunąć %d wybranych torrentów razem z danymi?
Are you sure to remove %d selected torrents?=Czy na pewno chcesz usunąć %d wybranych torrentów?
Bandwidth=Łącze
Directory for incomplete files=Folder na niedokończone pliki
Download=Pobierz
Enable blocklist=Włącz blocklistę
ID=Nr
Move torrent data from current location to new location=Przenieś dane tego torrenta do innej lokalizacji
New location for torrent data=Nowa lokalizacja na pobrane dane
No link was specified=Link nie został podany
No torrent location was specified=Nie została podana lokalizacja torrenta
Path=Ścieżka
Reannounce (get more peers)=Pobierz więcej peerów
Set data location=Ustaw lokalizację pobieranych plików
Size to download=Do pobrania
The block list has been updated successfully.~The list entries count: %d=Lista blokowanych adresów została pomyślnie zaktualizowana.~Liczba wpisów: %d
The directory for incomplete files was not specified=Katalog na niedokończone pliki nie został wskazany
The downloads directory was not specified=Katalog na pobierane pliki nie został wskazany
Torrent data location=Lokalizacja danych torrenta
Torrents=Torrenty
Unable to execute "%s"=Nie można uruchomić "%s"
Update blocklist=Uaktualnij blocklistę
URL of a .torrent file or a magnet link=URL pliku .torrent lub magnet linka
Columns setup=Ustawienia kolumn
Add torrent=Dodaj torrent
Delete a .torrent file after after a successful addition=Usuń plik .torrent po dodaniu
Torrent=Torrent
Torrents verification may take a long time.~Are you sure to start verification of %d torrents?=Weryfikacja torrentów może trwać bardzo długo.~Czy na pewno chcesz zweryfikować %d torrentów?
Unable to load OpenSSL library files: %s and %s=Nie można załadować biblioteki OpenSSL: %s i %s
Use SSL=Używaj SSL
Add tracker=Dodaj tracker
Alternate bandwidth settings=Alternatywne ustawienia łącza
Apply alternate bandwidth settings automatically=Automatycznie włącz alternatywne ustawienia łącza
Are you sure to delete connection '%s'?=Czy na pewno chcesz usunąć połączenie '%s'?
Are you sure to remove tracker '%s'?=Czy na pewno chcesz usunąć tracker '%s'?
Days=Dni
Delete=Usuń
Disk cache size=Wielkość cache dysku
Download speeds (KB/s)=Prędkości pobierania (KB/s)
Edit tracker=Edytuj tracker
Enable Local Peer Discovery=Włącz Local Peer Discovery
Free disk space=Wolne miejsce na dysku
Free: %s=Wolne: %s
From=Od
minutes=minut
Misc=Inne
New connection=Nowe połączenie
New=Nowy
No tracker URL was specified=URL trackera nie został podany
Proxy=Proxy
Remove tracker=Usuń tracker
Rename=Zmień nazwę
Speed limit menu items=Elementy menu ograniaczania prędkości
Stop seeding when inactive for=Zakończ seedowanie gdy nieaktywny przez
The invalid time value was entered=Został podany niepoprawny czas
to=do
Tracker announce URL=URL Trackera
Tracker properties=Właściwości trackera
Unlimited=Bez limitów
Upload speeds (KB/s)=Prędkości wysyłania (KB/s)
Use alternate bandwidth settings=Używaj alternatywnych ustawień łacza
average=średnio
Browse=Przeglądaj
Enable µTP=Włącz µTP
Select a folder for download=Wybierz folder pobierania
Select torrent location=Wybierz lokalizację torrenta
A new version of %s is available.~Your current version: %s~The new version: %s~~Do you wish to open the Downloads web page?=Nowa wersja %s jest dostępna.~Twoja obecna wersja: %s~Nowa wersja: %s~~Czy chcesz otworzyć stronę pobierania?
Advanced=Zaawansowane
Check for new version every=Sprawdź dostępność nowych wersji co
Check for updates=Sprawdź dostępność aktualizacji
Consider active torrents as stalled when idle for=Uznaj za wstrzymane, aktywne torrenty, gdy są bezczynne przez
Do you wish to enable automatic checking for a new version of %s?=Czy chcesz włączyć automatyczne sprawdzanie nowej wersji %s?
Donate!=Dotacje!
Download queue size=Rozmiar kolejki pobierania
Error checking for new version=Błąd sprawdzania nowej wersji 
Folder grouping=Grupowanie folderów
Force start=Wymuś start
Home page=Strona domowa
Modify trackers=Modyfikuj trackery
Move bottom=Przenieś na dół
Move down queue=Przenieś niżej w kolejce
Move down=Niżej
Move top=Przenieś na górę
Move up queue=Przenieś wyżej w kolejce
Move up=Wyżej
No updates have been found.~You are running the latest version of %s=Aktualizacja nie została znaleziona.~Używasz najnowszej wersji %s
Queue position=Pozycja w kolejce
Queue=Kolejka
Torrents that are idle for N minuets aren't counted toward the Download queue or Upload queue=Torrenty, które są bezczynne przez N minut, są pominięte z kolejki pobierania lub wysyłania
Tracker grouping=Grupowanie trackerów
Upload queue size=Rozmiar kolejki wysyłania
View=Widok
Visit home page=Odwiedź stronę domową
days=dni
Active time=Czas pracy
Automatically add torrent links from the clipboard=Automatycznie dodaj linki torrentów ze schowka
Copy file path to clipboard=Skopiuj ścieżkę pliku do schowka
Cumulative=Łączny
Current=Bieżący
Files added=Pliki dodane
Filter pane=Panel filtrów
Global statistics=Globalna statystyka
Info pane=Panel informacji
Statistics=Statystyka
Status bar=Pasek stanu
%dd=%dd
%dh=%dh
%dm=%dm
All torrents=All torrents
Application options=Application options
Ask for password=Ask for password
Authentication required=Authentication required
Average out transfer speeds to eliminate fluctuations=Average out transfer speeds to eliminate fluctuations
Connect to %s=Connect to %s
Connect to Transmission using proxy server=Connect to Transmission using proxy server
Connect to Transmission=Connect to Transmission
Connection name=Connection name
Could not connect to tracker==Could not connect to tracker
Data display=Data display
Data refresh interval when minimized=Data refresh interval when minimized
Data refresh interval=Data refresh interval
Default download folder on remote host=Default download folder on remote host
Disconnect from Transmission=Disconnect from Transmission
Downloading torrent file=Downloading torrent file
Font size=Font size
Handle .torrent files by %s=Handle .torrent files by %s
Handle magnet links by %s=Handle magnet links by %s
Invalid name specified=Invalid name specified
Manage connections to Transmission=Manage connections to Transmission
Manage connections=Manage connections
Network (WAN)=Network (WAN)
New connection to Transmission=New connection to Transmission
Pick random port on Transmission launch=Pick random port on Transmission launch
Please enter a password to connect to %s=Please enter a password to connect to %s
Please specify how %s will connect to a remote host running Transmission daemon (service)=Please specify how %s will connect to a remote host running Transmission daemon (service)
Prompt for download options when adding a new torrent=Prompt for download options when adding a new torrent
RPC path=RPC path
Save as=Save as
Seeding time=Seeding time
Show advanced options=Show advanced options
Show notifications in tray icon=Show notifications in tray icon
System integration=System integration
Torrent already exists in the list=Torrent already exists in the list
Torrent not registered with this tracker==Torrent not registered with this tracker
Unable to find path mapping.~Use the application's options to setup path mappings=Unable to find path mapping.~Use the application's options to setup path mappings
Update trackers for the existing torrent?=Update trackers for the existing torrent?
You need to restart the application to apply changes=You need to restart the application to apply changes
