TranslationLanguage=Česky

"Remote to local path mappings.~~Examples:~/share=\\pch\share~/var/downloads/music=Z:\music"="Mapování vzdálené cesty na lokalní.~~Příklady:~/share=\\pch\share~/var/downloads/music=Z:\music"
%d x %s (have %d)=%d x %s (hotových %d)
%ds=%ds
%s (%d hashfails)=%s (%d hash chyb)
%s (%s done)=%s (%s Hotovo)
'%s' has finished downloading='%s' dokončil stahování
%s%s%d downloading, %d seeding%s%s, %s=%s%s%d stahování, %d seedování%s%s, %s
&All=&Vše
&Help=&Nápověda
&Ignore=&Ignorovat
&No=&Ne
&OK=&OK
&Open=&Otevřít
&Retry=&Znovu
&Save=&Uložit
&Unlock=&Odemknout
&Yes=&Ano
/s=/s
Abort=Zrušit
About=O programu
Active=Aktivní
Add new torrent=Přidat nový torrent
&Add torrent=Přidat torrent
Added on=Přidáno
Are you sure to remove torrent '%s' and all associated DATA?=Opravdu chcete odstranit torrent '%s' a všechna s ním spojená DATA?
Are you sure to remove torrent '%s'?=Opravdu chcete odstranit torrent '%s'?
Authentication=Autentizace
b=b
Cancel=Zrušit
Client=Klient
Close to tray=Zavřít do tray
Comment=Komentář
Completed on=Dokončeno
Completed=Dokončeno
Confirmation=Potvrzení
connected=připojeno
Connecting to daemon=Připojování k démonovi
Connection error occurred=Chyba připojení
Copy=Kopírovat
Country=Stát
Created on=Vytvořeno
D: %s/s=D: %s/s
Destination folder=Cílový adresář
Disconnected=Odpojeno
Donate to support further development=Podpořte další vývoj
Donate via PayPal,WebMoney,Credit card=Darujte skrze PayPal,WebMoney,Credit card
Done=Hotovo
Don't download=Nestahovat
Down limit=Limit stahování
Down speed=Rychlost stahování
Down=Stahování
Download complete=Stahování dokončeno
Download speed=Rychlost stahování
Downloaded=Staženo
Downloading=Stahování
Enable DHT=Povolit DHT
Enable Peer Exchange=Povolit Peer Exchange
Enable port forwarding=Povolit port forwarding
Encryption disabled=Šifrování zakázáno
Encryption enabled=Šifrování povoleno
Encryption required=Šifrování požadováno
Encryption=Šifrování
Error=Chyba
ETA=Odhad času
E&xit=Ukončit
File name=Jméno souboru
Files=Soubory
Finished=Dokončeno
Flag images archive is needed to display country flags.~Download this archive now?=Archív vlajek je potřeba k zobrazení vlajek států.~Stáhnout nyní tento archív?
Flags=Vlajky
GB=GB
General=Hlavní
Geo IP database is needed to resolve country by IP address.~Download this database now?=Geo IP databáze je třeba k rozpoznání země podle IP adresy.~Stáhnout nyní tuto databázi?
Global bandwidth settings=Globální nastavní rychlostí
Global peer limit=Globální limit peerů
Hash=Hash
Have=Hotovo
Hide=Skrýt
High priority=Vysoká priorita
high=vysoká
Host=Host
in swarm=ve swarmu
Inactive=Neaktivní
Incoming port is closed. Check your firewall settings=Příchozí port je zavřený. Zkontrolujte vaše nastavení firewallu
Incoming port tested successfully=Příchozí port úspěšně otestován
Incoming port=Příchozí port
Information=Informace
KB/s=KB/s
KB=KB
Language=Jazyk
Last active=Naposledy aktivní
License=Licence
Low priority=Nízká priorita
low=nízká
Max peers=Maximum peerů
Maximum download speed=Maximální rychlost stahování
Maximum upload speed=Maximální rychlost uploadu
MB=MB
Minimize to tray=Minimalizovat do tray
Name=Jméno
No host name specified=Nenastevené jméno hosta
No proxy server specified=Nenastavnená proxy
No to all=Ne všem
Normal priority=Normální priorita
normal=normální
of=z
Open containing folder=Otevřít cílový adesář
Open=Otevřít
Password=Heslo
Paths=Cesty
Peer limit=Limit peerů
Peers=Peery
Pieces=Částí
Port=Port
Priority=Priorita
Properties=Vlastnosti
Proxy password=Heslo proxy
Proxy port=Port proxy
Proxy server=Proxy server
Proxy user name=Uživatelské jméno proxy
Ratio=Ratio
Reconnect in %d seconds=Znovupřipojení za %d sekund
Remaining=Zbývá
Remote host=Vzdálený host
Remove torrent and Data=Smazat torrent a data
Remove torrent=Smazat torrent
Remove=Smazat
Resolve country=Rozpoznat stát
Resolve host name=Rozpoznat hostname
seconds=sekund
Seed ratio=Ratio seedování
Seeding=Seedování
Seeds=Seedy
Select a .torrent to open=Vyberte .torrent k otevření
Select all=Označit vše
Select none=Odznačit vše
Setup columns=Nastavit sloupce
Share ratio=Poměr sdílení
Show country flag=Zobrazit vlajky států
Show=Zobrazit
Size=Velikost
skip=přeskočit
Start all torrents=Start všech torrentů
Start torrent=Start torrentu
Start=Start
Status=Status
Stop all torrents=Zastavit všechny torrenty
Stop all=Zastavit všechny
Stop torrent=Zastavit torrent
Stop=Zastavit
Stopped=Zastaveno
TB=TB
Test port=Test portu
T&ools=Nástroje
Torrent contents=Torrent contents
Torrent properties=Vlastnosti torrentu
Torrent verification may take a long time.~Are you sure to start verification of torrent '%s'?=Ověření torrentu může nějkou doby trvat.~Opravdu chcete ověřit torrent '%s'?
&Torrent=Torrent
Torrents (*.torrent)|*.torrent|All files (*.*)|*.*=Torrenty (*.torrent)|*.torrent|Všechny soubory (*.*)|*.*
Total size=Celková velikost
Tracker status=Status trackeru
Tracker update on=Update trackeru za
Tracker=Tracker
Trackers=Trackery
Transfer=Přenos
Transmission options=Možnosti transmission
Transmission%s at %s:%s=Transmission%s v %s:%s
Tray icon always visible=Tray ikona stále zobrazená
Tray icon=Tray ikona
U: %s/s=U: %s/s
Unable to extract flag image=Není možné rozbalit obrázek vlajky
Unable to get files list=Není možné získat seznam souborů
Unknown=Neznámo
Up limit=Limit uploadu
Up speed=Rychlost uploadu
Up=Up
Update complete=Obnovení dokončeno
Update GeoIP database=Obnovit databázi GeoIP
Update in=Obnovení za
Updating=Obnovuji
Upload speed=Rychlost uploadu
Uploaded=Uploadováno
User name=Uživatelské jméno
Verify torrent=Ověřit torrent
&Verify=Ověřit
Verifying=Ověřuji
Version %s=Verze %s
Waiting=Čekání
Warning=Varování
Wasted=Zahozeno
Working=Běží
Yes to &All=Ano &Všem
No tracker=Žádný tracker
%s downloaded=%s staženo
%s of %s downloaded=%s z %s staženo
%d torrents=%d torrentů
Add .part extension to incomplete files=Přidat příponu .part nedokončeným souborům
Add torrent link=Přidat URL
Are you sure to remove %d selected torrents and all their associated DATA?=Opravdu chcete odstranit %d vybraných torrentů a všechna jejich DATA?
Are you sure to remove %d selected torrents?=Opravdu chcete odstranit %d vybraných torrentů?
Bandwidth=Rychlost
Directory for incomplete files=Adresář pro nedokončené soubory
Download=Stažené
Enable blocklist=Povolit blocklist
ID=ID
Move torrent data from current location to new location=Přesunout data torrentu z aktuálního umístění na nové
New location for torrent data=Nové umístění pro data torrentu
No link was specified=Není uveden žádný link
No torrent location was specified=Není uvedeno žádné umístění
Path=Cesta
Reannounce (get more peers)=Obnovit (získá více peerů)
Set data location=Nastavit umístění dat
Size to download=Velikost ke stažení
The block list has been updated successfully.~The list entries count: %d=Blocklist úspěšne aktualizován.~Počet položek: %d
The directory for incomplete files was not specified=Adresář pro nedokončené soubory není uveden
The downloads directory was not specified=Adresář pro stahování není uveden
Torrent data location=Umístění dat torrentu
Torrents=Torrenty
Unable to execute "%s"=Není možné spustit "%s"
Update blocklist=Aktualizovat blocklist
URL of a .torrent file or a magnet link=URL .torrent souboru nebo magnet link
Columns setup=Nastavení sloupců
Add torrent=Přidat torrent
Delete a .torrent file after after a successful addition=Odstranit .torrent soubor po úspěšném přidání
Torrent=Torrent
Torrents verification may take a long time.~Are you sure to start verification of %d torrents?=Ověřování torrentu může trvat delší dobu.~Opravdu chcete ověřit %d torrentů?
Unable to load OpenSSL library files: %s and %s=Nebylo možné otevřít soubory knihovny OpenSSL: %s a %s
Use SSL=Použít SSL
Add tracker=Přidat tracker
Alternate bandwidth settings=Alternativní nastavení rychlostí
Apply alternate bandwidth settings automatically=Automaticky použít alternativní nastavení rychlostí
Are you sure to delete connection '%s'?=Opravdu chcete smazat spojení '%s'?
Are you sure to remove tracker '%s'?=Opravdu chcete odstranit tracker '%s'?
Days=Dny
Delete=Smazat
Disk cache size=Velikost vyrovnávací paměti
Download speeds (KB/s)=Rychlost stahování (KB/s)
Edit tracker=Upravit tracker
Enable Local Peer Discovery=Povolit Hledání Místních Peerů
Free disk space=Volné místo na disku
Free: %s=Volno: %s
From=Od
minutes=minut
Misc=Další
New connection=Nové spojení
New=Nové
No tracker URL was specified=Nebyla zadána URL trackeru
Proxy=Proxy
Remove tracker=Odstranit tracker
Rename=Přejmenovat
Speed limit menu items=Položky menu omezení rychlosti
Stop seeding when inactive for=Přestat seedovat pokud je neaktivní déle jak 
The invalid time value was entered=Byl zadán chybný časový údaj
to=do
Tracker announce URL=Announce URL trackeru
Tracker properties=Vlastnosti trackeru
Unlimited=Neomezeně
Upload speeds (KB/s)=Rychlost uploadu (KB/s)
Use alternate bandwidth settings=Použít alternativní nastavené rychlostí
average=average
Browse=Procházet
Enable µTP=Povolit µTP
Select a folder for download=Vybrat adresář pro stahování
Select torrent location=Vybrat umístění torrentu
&Close=&Zavřít
A new version of %s is available.~Your current version: %s~The new version: %s~~Do you wish to open the Downloads web page?=Je dostupná nová verze %s.~Současná verze: %s~Nová verze: %s~~Chtěli byste otevřít stránky se stažením?
Advanced=Pokročilé
Check for new version every=Kontrolovat nové verze jednou za
Check for updates=Kontrola updatů
Consider active torrents as stalled when idle for=Považovat aktivní torrenty za pozastavené pokud jsou nečinné
Do you wish to enable automatic checking for a new version of %s?=Chtěli byste povolit automatickou kontrolu nových verzí %s?
Donate!=Podpořte!
Download queue size=Velikost fronty stahování
Error checking for new version=Chyba při kontrole nové verze
Folder grouping=Filtr adresářů
Force start=Vynucený start
Home page=Domovská stránka
Modify trackers=Upravit trackery
Move bottom=Posunout dospod
Move down queue=Posunout frontou dolů
Move down=Posunout dolů
Move top=Posunout navrch
Move up queue=Posunout frontou nahoru
Move up=Posunout nahoru
No updates have been found.~You are running the latest version of %s=Žádné updaty nenalezeny~Používáte nejnovější verzi %s
Queue position=Pozice ve frontě
Queue=Fronta
Torrents that are idle for N minuets aren't counted toward the Download queue or Upload queue=Torrenty nečinné alespoň N minut se nezpočítavají do Stahovací nebo Odesílací fronty
Tracker grouping=Filtr trackerů
Upload queue size=Velikost fronty odesílání
View=Zobrazit
Visit home page=Navštívit domovskou stránku
days=dny
Active time=Doba běhu
Automatically add torrent links from the clipboard=Automaticky přidávat torrent linky ze schránky
Copy file path to clipboard=Kopírovat cestu k souboru do schránky
Cumulative=Cumulative
Current=Současný
Files added=Přidáno souborů
Filter pane=Filtry
Global statistics=Globální statistiky
Info pane=Informace
Statistics=Statistiky
Status bar=Stavový řádek
%dd=%dd
%dh=%dh
%dm=%dm
All torrents=Všechny torrenty
Application options=Možnosti aplikace
Ask for password=Zeptat se na heslo
Authentication required=Vyžadovat ověření
Average out transfer speeds to eliminate fluctuations=Zprůměrovat rychlosti přenosu k odstranění kolísání
Connect to %s=Připojit k %s
Connect to Transmission using proxy server=Připojit k Transmission pomocí proxy serveru
Connect to Transmission=Připojit k Transmission
Connection name=Jméno spojení
Could not connect to tracker=Nelze se spojit s trackerem
Data display=Zobrazení informací
Data refresh interval when minimized=Interval obnovení informací při minimalizaci
Data refresh interval=Interval obnovení informací
Default download folder on remote host=Výchozí složka pro stahování na vzdáleném hostiteli
Disconnect from Transmission=Odpojit od Transmission
Downloading torrent file=Stahuji torrent soubor
Font size=Velikost fontu
Handle .torrent files by %s=Otevírat .torrent soubory pomocí %s
Handle magnet links by %s=Otevírat magnet linky pomocí %s
Invalid name specified=Zadáno neplatné jméno
Manage connections to Transmission=Spravovat spojení s Transmission
Manage connections=Správa spojení
Network (WAN)=Síť (WAN)
New connection to Transmission=Nové spojení s Transmission
Pick random port on Transmission launch=Vybrat náhodný port při spuštění Transmission
Please enter a password to connect to %s=Pro spojení s %s prosím vložte heslo
Please specify how %s will connect to a remote host running Transmission daemon (service)=Určete jak se bude %s připojovat ke vzdálenému hostiteli s běžícím demonem (službou) Transmission
Prompt for download options when adding a new torrent=Dotázat se na možnosti stahováni při přidání nového torrentu
RPC path=cesta RPC
Save as=Uložit jako
Seeding time=Doba seedování
Show advanced options=Zobrazit pokročilé možnosti
Show notifications in tray icon=Zobrazit notifikace v tray ikoně
System integration=Systémová integrace
Torrent already exists in the list=Torrent už je v seznamu
Torrent not registered with this tracker=Torrent není na tomto trackeru registrován
Unable to find path mapping.~Use the application's options to setup path mappings=Není možné najít mapování cest. ~Nastavte mapování cest přes možnosti aplikace
Update trackers for the existing torrent?=Aktualizovat trackery pro existující torrent?
You need to restart the application to apply changes=Aby se projevily změny, je nutné aplikaci restartovat
